package javax.swing.text.html;

import javax.swing.DefaultButtonModel;
import javax.swing.text.PlainDocument;
import javax.swing.JToggleButton;
import java.net.MalformedURLException;
import sun.swing.SwingUtilities2;
import java.awt.font.TextAttribute;
import java.util.Stack;
import java.util.Vector;
import javax.swing.text.Document;
import javax.swing.undo.UndoableEdit;
import java.io.StringReader;
import java.util.Enumeration;
import java.util.Hashtable;
import java.io.IOException;
import javax.swing.text.ElementIterator;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.Element;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.DocumentEvent;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.BadLocationException;
import javax.swing.text.AbstractDocument;
import javax.swing.text.GapContent;
import javax.swing.text.AttributeSet;
import java.net.URL;
import javax.swing.ButtonGroup;
import java.util.HashMap;
import javax.swing.text.DefaultStyledDocument;

public class HTMLDocument extends DefaultStyledDocument
{
    private boolean frameDocument;
    private boolean preservesUnknownTags;
    private HashMap<String, ButtonGroup> radioButtonGroupsMap;
    static final String TokenThreshold = "token threshold";
    private static final int MaxThreshold = 10000;
    private static final int StepThreshold = 5;
    public static final String AdditionalComments = "AdditionalComments";
    static final String StyleType = "StyleType";
    URL base;
    boolean hasBaseTag;
    private String baseTarget;
    private HTMLEditorKit.Parser parser;
    private static AttributeSet contentAttributeSet;
    static String MAP_PROPERTY;
    private static char[] NEWLINE;
    private boolean insertInBody;
    private static final String I18NProperty = "i18n";
    
    public HTMLDocument() {
        this(new GapContent(4096), new StyleSheet());
    }
    
    public HTMLDocument(StyleSheet styleSheet) {
        this(new GapContent(4096), styleSheet);
    }
    
    public HTMLDocument(AbstractDocument.Content c, StyleSheet styles) {
        super(c, styles);
        this.frameDocument = false;
        this.preservesUnknownTags = true;
        this.hasBaseTag = false;
        this.baseTarget = null;
        this.insertInBody = false;
    }
    
    public HTMLEditorKit.ParserCallback getReader(int n) {
        Object property = this.getProperty("stream");
        if ((property instanceof URL)) {
            this.setBase((URL)property);
        }
        return new HTMLDocument.HTMLReader(n);
    }
    
    public HTMLEditorKit.ParserCallback getReader(int n, int n2, int n3, HTML.Tag tag) {
        return this.getReader(n, n2, n3, tag, true);
    }
    
    HTMLEditorKit.ParserCallback getReader(int n, int n2, int n3, HTML.Tag tag, boolean b) {
        Object property = this.getProperty("stream");
        if ((property instanceof URL)) {
            this.setBase((URL)property);
        }
        return new HTMLDocument.HTMLReader(n, n2, n3, tag, b, false, true);
    }
    
    public URL getBase() {
        return this.base;
    }
    
    public void setBase(URL url) {
        this.base = url;
        this.getStyleSheet().setBase(url);
    }
    
    @Override
    protected void insert(int offset, DefaultStyledDocument.ElementSpec[] data) throws BadLocationException {
        super.insert(offset, data);
    }
    
    @Override
    protected void insertUpdate(AbstractDocument.DefaultDocumentEvent chng, AttributeSet contentAttributeSet) {
        if (contentAttributeSet == null) {
            contentAttributeSet = HTMLDocument.contentAttributeSet;
        }
        else if (contentAttributeSet.isDefined(StyleConstants.ComposedTextAttribute)) {
            ((MutableAttributeSet)contentAttributeSet).addAttributes(HTMLDocument.contentAttributeSet);
        }
        if (contentAttributeSet.isDefined("CR")) {
            ((MutableAttributeSet)contentAttributeSet).removeAttribute("CR");
        }
        super.insertUpdate(chng, contentAttributeSet);
    }
    
    @Override
    protected void create(DefaultStyledDocument.ElementSpec[] data) {
        super.create(data);
    }
    
    @Override
    public void setParagraphAttributes(int i, int max, AttributeSet set, boolean isReplacing) {
        try {
            this.writeLock();
            int min = Math.min(i + max, this.getLength());
            i = this.getParagraphElement(i).getStartOffset();
            max = Math.max(0, this.getParagraphElement(min).getEndOffset() - i);
            AbstractDocument.DefaultDocumentEvent edit = new AbstractDocument.DefaultDocumentEvent(i, max, DocumentEvent.EventType.CHANGE);
            AttributeSet copyAttributes = set.copyAttributes();
            int endOffset;
            Element paragraphElement;
            MutableAttributeSet set2;
            for (endOffset = Integer.MAX_VALUE, ; i <= min; i = endOffset) {
                paragraphElement = this.getParagraphElement(i);
                if (endOffset == paragraphElement.getEndOffset()) {
                    ++endOffset;
                }
                else {
                    endOffset = paragraphElement.getEndOffset();
                }
                set2 = (MutableAttributeSet)paragraphElement.getAttributes();
                edit.addEdit(new DefaultStyledDocument.AttributeUndoableEdit(paragraphElement, copyAttributes, isReplacing));
                if (isReplacing) {
                    set2.removeAttributes(set2);
                }
                set2.addAttributes(set);
            }
            edit.end();
            this.fireChangedUpdate(edit);
            this.fireUndoableEditUpdate(new UndoableEditEvent(this, edit));
        }
        finally {
            this.writeUnlock();
        }
    }
    
    public StyleSheet getStyleSheet() {
        return (StyleSheet)this.getAttributeContext();
    }
    
    public HTMLDocument.Iterator getIterator(HTML.Tag tag) {
        if (tag.isBlock()) {
            return null;
        }
        return new HTMLDocument.LeafIterator(tag, this);
    }
    
    @Override
    protected Element createLeafElement(Element element, AttributeSet set, int n, int n2) {
        return new HTMLDocument.RunElement(element, set, n, n2);
    }
    
    @Override
    protected Element createBranchElement(Element element, AttributeSet set) {
        return new HTMLDocument.BlockElement(element, set);
    }
    
    @Override
    protected AbstractDocument.AbstractElement createDefaultRoot() {
        this.writeLock();
        SimpleAttributeSet set = new SimpleAttributeSet();
        set.addAttribute(StyleConstants.NameAttribute, HTML.Tag.HTML);
        HTMLDocument.BlockElement blockElement = new HTMLDocument.BlockElement(null, set.copyAttributes());
        set.removeAttributes(set);
        set.addAttribute(StyleConstants.NameAttribute, HTML.Tag.BODY);
        HTMLDocument.BlockElement blockElement2 = new HTMLDocument.BlockElement(blockElement, set.copyAttributes());
        set.removeAttributes(set);
        set.addAttribute(StyleConstants.NameAttribute, HTML.Tag.P);
        this.getStyleSheet().addCSSAttributeFromHTML(set, CSS.Attribute.MARGIN_TOP, "0");
        HTMLDocument.BlockElement blockElement3 = new HTMLDocument.BlockElement(blockElement2, set.copyAttributes());
        set.removeAttributes(set);
        set.addAttribute(StyleConstants.NameAttribute, HTML.Tag.CONTENT);
        Element[] elems = { new HTMLDocument.RunElement(blockElement3, set, 0, 1) };
        blockElement3.replace(0, 0, elems);
        elems[0] = blockElement3;
        blockElement2.replace(0, 0, elems);
        elems[0] = blockElement2;
        blockElement.replace(0, 0, elems);
        this.writeUnlock();
        return blockElement;
    }
    
    public void setTokenThreshold(int value) {
        this.putProperty("token threshold", new Integer(value));
    }
    
    public int getTokenThreshold() {
        Integer n = (Integer)this.getProperty("token threshold");
        if (n != null) {
            return (int)n;
        }
        return Integer.MAX_VALUE;
    }
    
    public void setPreservesUnknownTags(boolean preservesUnknownTags) {
        this.preservesUnknownTags = preservesUnknownTags;
    }
    
    public boolean getPreservesUnknownTags() {
        return this.preservesUnknownTags;
    }
    
    public void processHTMLFrameHyperlinkEvent(HTMLFrameHyperlinkEvent htmlFrameHyperlinkEvent) {
        String target = htmlFrameHyperlinkEvent.getTarget();
        Element sourceElement = htmlFrameHyperlinkEvent.getSourceElement();
        String string = htmlFrameHyperlinkEvent.getURL().toString();
        if (target.equals("_self")) {
            this.updateFrame(sourceElement, string);
        }
        else if (target.equals("_parent")) {
            this.updateFrameSet(sourceElement.getParentElement(), string);
        }
        else {
            Element frame = this.findFrame(target);
            if (frame != null) {
                this.updateFrame(frame, string);
            }
        }
    }
    
    private Element findFrame(String anObject) {
        Element next;
        while ((next = new ElementIterator(this).next()) != null) {
            AttributeSet attributes = next.getAttributes();
            if (matchNameAttribute(attributes, HTML.Tag.FRAME)) {
                String s = (String)attributes.getAttribute(HTML.Attribute.NAME);
                if (s != null && s.equals(anObject)) {
                    break;
                }
                continue;
            }
            else {
                continue;
            }
        }
        return next;
    }
    
    static boolean matchNameAttribute(AttributeSet set, HTML.Tag tag) {
        Object attribute = set.getAttribute(StyleConstants.NameAttribute);
        return attribute instanceof HTML.Tag && (HTML.Tag)attribute == tag;
    }
    
    private void updateFrameSet(Element element, String str) {
        try {
            element.getStartOffset();
            Math.min(this.getLength(), element.getEndOffset());
            String string = "<frame";
            if (str != null) {
                string = string + " src=\"" + str + "\"";
            }
            String string2 = string + ">";
            this.installParserIfNecessary();
            this.setOuterHTML(element, string2);
        }
        catch (BadLocationException ex) {}
        catch (IOException ex2) {}
    }
    
    private void updateFrame(Element element, String s) {
        try {
            this.writeLock();
            AbstractDocument.DefaultDocumentEvent edit = new AbstractDocument.DefaultDocumentEvent(element.getStartOffset(), 1, DocumentEvent.EventType.CHANGE);
            AttributeSet copyAttributes = element.getAttributes().copyAttributes();
            MutableAttributeSet set = (MutableAttributeSet)element.getAttributes();
            edit.addEdit(new DefaultStyledDocument.AttributeUndoableEdit(element, copyAttributes, false));
            set.removeAttribute(HTML.Attribute.SRC);
            set.addAttribute(HTML.Attribute.SRC, s);
            edit.end();
            this.fireChangedUpdate(edit);
            this.fireUndoableEditUpdate(new UndoableEditEvent(this, edit));
        }
        finally {
            this.writeUnlock();
        }
    }
    
    boolean isFrameDocument() {
        return this.frameDocument;
    }
    
    void setFrameDocumentState(boolean frameDocument) {
        this.frameDocument = frameDocument;
    }
    
    void addMap(Map value) {
        String name = value.getName();
        if (name != null) {
            Object property = this.getProperty(HTMLDocument.MAP_PROPERTY);
            if (property == null) {
                property = new Hashtable(11);
                this.putProperty(HTMLDocument.MAP_PROPERTY, property);
            }
            if ((property instanceof Hashtable)) {
                ((Hashtable)property).put("#" + name, value);
            }
        }
    }
    
    void removeMap(Map map) {
        String name = map.getName();
        if (name != null) {
            Object property = this.getProperty(HTMLDocument.MAP_PROPERTY);
            if ((property instanceof Hashtable)) {
                ((Hashtable)property).remove("#" + name);
            }
        }
    }
    
    Map getMap(String key) {
        if (key != null) {
            Object property = this.getProperty(HTMLDocument.MAP_PROPERTY);
            if (property != null && property instanceof Hashtable) {
                return (Map)((Hashtable)property).get(key);
            }
        }
        return null;
    }
    
    Enumeration getMaps() {
        Object property = this.getProperty(HTMLDocument.MAP_PROPERTY);
        if ((property instanceof Hashtable)) {
            return ((Hashtable)property).elements();
        }
        return null;
    }
    
    void setDefaultStyleSheetType(String value) {
        this.putProperty("StyleType", value);
    }
    
    String getDefaultStyleSheetType() {
        String s = (String)this.getProperty("StyleType");
        if (s == null) {
            return "text/css";
        }
        return s;
    }
    
    public void setParser(HTMLEditorKit.Parser parser) {
        this.parser = parser;
        this.putProperty("__PARSER__", null);
    }
    
    public HTMLEditorKit.Parser getParser() {
        Object property = this.getProperty("__PARSER__");
        if ((property instanceof HTMLEditorKit.Parser)) {
            return (HTMLEditorKit.Parser)property;
        }
        return this.parser;
    }
    
    public void setInnerHTML(Element element, String s) throws BadLocationException, IOException {
        this.verifyParser();
        if (element != null && element.isLeaf()) {
            throw new IllegalArgumentException("Can not set inner HTML of a leaf");
        }
        if (element != null && s != null) {
            int elementCount = element.getElementCount();
            element.getStartOffset();
            this.insertHTML(element, element.getStartOffset(), s, true);
            if (element.getElementCount() > elementCount) {
                this.removeElements(element, element.getElementCount() - elementCount, elementCount);
            }
        }
        return;
    }
    
    public void setOuterHTML(Element element, String s) throws BadLocationException, IOException {
        this.verifyParser();
        if (element != null && element.getParentElement() != null && s != null) {
            int startOffset = element.getStartOffset();
            int endOffset = element.getEndOffset();
            int length = this.getLength();
            int n = !element.isLeaf();
            if (n == 0 && (endOffset > length || this.getText(endOffset - 1, 1).charAt(0) == HTMLDocument.NEWLINE[0])) {
                n = true;
            }
            Element parentElement = element.getParentElement();
            int elementCount = parentElement.getElementCount();
            this.insertHTML(parentElement, startOffset, s, (boolean)n);
            int length2 = this.getLength();
            if (elementCount != parentElement.getElementCount()) {
                this.removeElements(parentElement, parentElement.getElementIndex(startOffset + length2 - length), 1);
            }
        }
    }
    
    public void insertAfterStart(Element element, String s) throws BadLocationException, IOException {
        this.verifyParser();
        if (element == null || s == null) {
            return;
        }
        if (element.isLeaf()) {
            throw new IllegalArgumentException("Can not insert HTML after start of a leaf");
        }
        this.insertHTML(element, element.getStartOffset(), s, false);
        return;
    }
    
    public void insertBeforeEnd(Element element, String s) throws BadLocationException, IOException {
        this.verifyParser();
        if (element != null && element.isLeaf()) {
            throw new IllegalArgumentException("Can not set inner HTML before end of leaf");
        }
        if (element != null) {
            int endOffset = element.getEndOffset();
            if (element.getElement(element.getElementIndex(endOffset - 1)).isLeaf() && this.getText(endOffset - 1, 1).charAt(0) == HTMLDocument.NEWLINE[0]) {
                --endOffset;
            }
            this.insertHTML(element, endOffset, s, false);
        }
        return;
    }
    
    public void insertBeforeStart(Element element, String s) throws BadLocationException, IOException {
        this.verifyParser();
        if (element != null) {
            Element parentElement = element.getParentElement();
            if (parentElement != null) {
                this.insertHTML(parentElement, element.getStartOffset(), s, false);
            }
        }
    }
    
    public void insertAfterEnd(Element element, String s) throws BadLocationException, IOException {
        this.verifyParser();
        if (element != null) {
            Element parentElement = element.getParentElement();
            if (parentElement != null) {
                if (HTML.Tag.BODY.name.equals(parentElement.getName())) {
                    this.insertInBody = true;
                }
                int endOffset = element.getEndOffset();
                if (endOffset > this.getLength() + 1) {
                    --endOffset;
                }
                else if (element.isLeaf() && this.getText(endOffset - 1, 1).charAt(0) == HTMLDocument.NEWLINE[0]) {
                    --endOffset;
                }
                this.insertHTML(parentElement, endOffset, s, false);
                if (this.insertInBody) {
                    this.insertInBody = false;
                }
            }
        }
    }
    
    public Element getElement(String s) {
        if (s == null) {
            return null;
        }
        return this.getElement(this.getDefaultRootElement(), HTML.Attribute.ID, s, true);
    }
    
    public Element getElement(Element element, Object o, Object o2) {
        return this.getElement(element, o, o2, true);
    }
    
    private Element getElement(Element element, Object o, Object o2, boolean b) {
        AttributeSet attributes = element.getAttributes();
        if (attributes != null && attributes.isDefined(o) && o2.equals(attributes.getAttribute(o))) {
            return element;
        }
        if (!element.isLeaf()) {
            int i = 0;
            while (i < element.getElementCount()) {
                Element element2 = this.getElement(element.getElement(i), o, o2, b);
                if (element2 != null) {
                    return element2;
                }
                ++i;
            }
        }
        else if (b && attributes != null) {
            Enumeration<Object> attributeNames = attributes.getAttributeNames();
            if (attributeNames != null) {
                while (attributeNames.hasMoreElements()) {
                    Object nextElement = attributeNames.nextElement();
                    if (nextElement instanceof HTML.Tag && attributes.getAttribute(nextElement) instanceof AttributeSet) {
                        AttributeSet set = (AttributeSet)attributes.getAttribute(nextElement);
                        if (set.isDefined(o) && o2.equals(set.getAttribute(o))) {
                            return element;
                        }
                        continue;
                    }
                    else {
                        continue;
                    }
                }
            }
        }
        return null;
    }
    
    private void verifyParser() {
        if (this.getParser() == null) {
            throw new IllegalStateException("No HTMLEditorKit.Parser");
        }
        return;
    }
    
    private void installParserIfNecessary() {
        if (this.getParser() == null) {
            this.setParser(new HTMLEditorKit().getParser());
        }
    }
    
    private void insertHTML(Element parentElement, int offs, String s, boolean b) throws BadLocationException, IOException {
        if (parentElement != null && s != null) {
            HTMLEditorKit.Parser parser = this.getParser();
            if (parser != null) {
                int max = Math.max(0, offs - 1);
                Element element = this.getCharacterElement(max);
                int n = 0;
                int n2 = 0;
                if (parentElement.getStartOffset() > max) {
                    while (parentElement != null && parentElement.getStartOffset() > max) {
                        parentElement = parentElement.getParentElement();
                        ++n2;
                    }
                    if (parentElement == null) {
                        throw new BadLocationException("No common parent", offs);
                    }
                }
                while (element != null && element != parentElement) {
                    ++n;
                    element = element.getParentElement();
                }
                if (element != null) {
                    HTMLDocument.HTMLReader o = new HTMLDocument.HTMLReader(offs, n - 1, n2, null, false, true, b);
                    parser.parse(new StringReader(s), o, true);
                    o.flush();
                }
            }
        }
    }
    
    private void removeElements(Element element, int n, int n2) throws BadLocationException {
        this.writeLock();
        try {
            int startOffset = element.getElement(n).getStartOffset();
            int endOffset = element.getElement(n + n2 - 1).getEndOffset();
            if (endOffset > this.getLength()) {
                this.removeElementsAtEnd(element, n, n2, startOffset, endOffset);
            }
            else {
                this.removeElements(element, n, n2, startOffset, endOffset);
            }
        }
        finally {
            this.writeUnlock();
        }
    }
    
    private void removeElementsAtEnd(Element element, int n, int n2, int n3, int n4) throws BadLocationException {
        int leaf = element.getElement(n - 1).isLeaf();
        AbstractDocument.DefaultDocumentEvent edit = new AbstractDocument.DefaultDocumentEvent(n3 - 1, n4 - n3 + 1, DocumentEvent.EventType.REMOVE);
        if (!(leaf == 0)) {
            Element characterElement = this.getCharacterElement(this.getLength());
            --n;
            if (characterElement.getParentElement() != element) {
                this.replace(edit, element, n, ++n2, n3, n4, true, true);
            }
            else {
                this.replace(edit, element, n, n2, n3, n4, true, false);
            }
        }
        else {
            Element element2;
            for (element2 = element.getElement(n - 1); !element2.isLeaf(); element2 = element2.getElement(element2.getElementCount() - 1)) {}
            Element parentElement = element2.getParentElement();
            this.replace(edit, element, n, n2, n3, n4, false, false);
            this.replace(edit, parentElement, parentElement.getElementCount() - 1, 1, n3, n4, true, true);
        }
        this.postRemoveUpdate(edit);
        edit.end();
        this.fireRemoveUpdate(edit);
        this.fireUndoableEditUpdate(new UndoableEditEvent(this, edit));
    }
    
    private void replace(AbstractDocument.DefaultDocumentEvent defaultDocumentEvent, Element e, int n, int n2, int n3, int n4, boolean b, boolean b2) throws BadLocationException {
        AttributeSet attributes = e.getElement(n).getAttributes();
        Element[] removed = new Element[n2];
        for (int i = 0; i < n2; ++i) {
            removed[i] = e.getElement(i + n);
        }
        if (b) {
            UndoableEdit remove = this.getContent().remove(n3 - 1, n4 - n3);
            if (remove != null) {
                defaultDocumentEvent.addEdit(remove);
            }
        }
        Element[] array;
        if (b2) {
            array = new Element[] { this.createLeafElement(e, attributes, n3 - 1, n3) };
        }
        else {
            array = new Element[0];
        }
        defaultDocumentEvent.addEdit(new AbstractDocument.ElementEdit(e, n, removed, array));
        ((AbstractDocument.BranchElement)e).replace(n, removed.length, array);
    }
    
    private void removeElements(Element e, int n, int n2, int offs, int n3) throws BadLocationException {
        Element[] removed = new Element[n2];
        Element[] array = new Element[0];
        for (int i = 0; i < n2; ++i) {
            removed[i] = e.getElement(i + n);
        }
        AbstractDocument.DefaultDocumentEvent edit = new AbstractDocument.DefaultDocumentEvent(offs, n3 - offs, DocumentEvent.EventType.REMOVE);
        ((AbstractDocument.BranchElement)e).replace(n, removed.length, array);
        edit.addEdit(new AbstractDocument.ElementEdit(e, n, removed, array));
        UndoableEdit remove = this.getContent().remove(offs, n3 - offs);
        if (remove != null) {
            edit.addEdit(remove);
        }
        this.postRemoveUpdate(edit);
        edit.end();
        this.fireRemoveUpdate(edit);
        if (remove != null) {
            this.fireUndoableEditUpdate(new UndoableEditEvent(this, edit));
        }
    }
    
    void obtainLock() {
        this.writeLock();
    }
    
    void releaseLock() {
        this.writeUnlock();
    }
    
    @Override
    protected void fireChangedUpdate(DocumentEvent e) {
        super.fireChangedUpdate(e);
    }
    
    @Override
    protected void fireUndoableEditUpdate(UndoableEditEvent e) {
        super.fireUndoableEditUpdate(e);
    }
    
    boolean hasBaseTag() {
        return this.hasBaseTag;
    }
    
    String getBaseTarget() {
        return this.baseTarget;
    }
    
    static void access$200(HTMLDocument htmlDocument, AbstractDocument.DefaultDocumentEvent chng) {
        htmlDocument.removeUpdate(chng);
    }
    
    static AbstractDocument.Content access$300(HTMLDocument htmlDocument) {
        return htmlDocument.getContent();
    }
    
    static void access$400(HTMLDocument htmlDocument, AbstractDocument.DefaultDocumentEvent chng) {
        htmlDocument.postRemoveUpdate(chng);
    }
    
    static void access$500(HTMLDocument htmlDocument, DocumentEvent e) {
        htmlDocument.fireRemoveUpdate(e);
    }
    
    static {
        HTMLDocument.MAP_PROPERTY = "__MAP__";
        HTMLDocument.contentAttributeSet = new SimpleAttributeSet();
        ((MutableAttributeSet)HTMLDocument.contentAttributeSet).addAttribute(StyleConstants.NameAttribute, HTML.Tag.CONTENT);
        (HTMLDocument.NEWLINE = new char[1])[0] = '\n';
    }
    
    public abstract static class Iterator
    {
        public abstract AttributeSet getAttributes();
        
        public abstract int getStartOffset();
        
        public abstract int getEndOffset();
        
        public abstract void next();
        
        public abstract boolean isValid();
        
        public abstract HTML.Tag getTag();
    }
    
    static class LeafIterator extends HTMLDocument.Iterator
    {
        private int endOffset;
        private HTML.Tag tag;
        private ElementIterator pos;
        
        LeafIterator(HTML.Tag tag, Document document) {
            this.tag = tag;
            this.pos = new ElementIterator(document);
            this.endOffset = 0;
            this.next();
        }
        
        @Override
        public AttributeSet getAttributes() {
            Element current = this.pos.current();
            if (current != null) {
                AttributeSet attributes = (AttributeSet)current.getAttributes().getAttribute(this.tag);
                if (attributes == null) {
                    attributes = current.getAttributes();
                    return attributes;
                }
                return attributes;
            }
            else {
                return null;
            }
        }
        
        @Override
        public int getStartOffset() {
            Element current = this.pos.current();
            if (current != null) {
                return current.getStartOffset();
            }
            return -1;
        }
        
        @Override
        public int getEndOffset() {
            return this.endOffset;
        }
        
        @Override
        public void next() {
            this.nextLeaf(this.pos);
            while (this.isValid()) {
                if (this.pos.current().getStartOffset() >= this.endOffset) {
                    AttributeSet attributes = this.pos.current().getAttributes();
                    if (attributes.isDefined(this.tag) || attributes.getAttribute(StyleConstants.NameAttribute) == this.tag) {
                        this.setEndOffset();
                        break;
                    }
                }
                this.nextLeaf(this.pos);
            }
        }
        
        @Override
        public HTML.Tag getTag() {
            return this.tag;
        }
        
        @Override
        public boolean isValid() {
            return this.pos.current() != null;
        }
        
        void nextLeaf(ElementIterator elementIterator) {
            elementIterator.next();
            while (elementIterator.current() != null && !elementIterator.current().isLeaf()) {
                elementIterator.next();
            }
        }
        
        void setEndOffset() {
            AttributeSet attributes = this.getAttributes();
            this.endOffset = this.pos.current().getEndOffset();
            ElementIterator elementIterator = (ElementIterator)this.pos.clone();
            this.nextLeaf(elementIterator);
            while (elementIterator.current() != null) {
                Element current = elementIterator.current();
                AttributeSet set = (AttributeSet)current.getAttributes().getAttribute(this.tag);
                if (set != null) {
                    if (!set.equals(attributes)) {
                        break;
                    }
                    this.endOffset = current.getEndOffset();
                    this.nextLeaf(elementIterator);
                }
                else {
                    break;
                }
            }
        }
    }
    
    public class HTMLReader extends HTMLEditorKit.ParserCallback
    {
        private boolean receivedEndHTML;
        private int flushCount;
        private boolean insertAfterImplied;
        private boolean wantsTrailingNewline;
        int threshold;
        int offset;
        boolean inParagraph;
        boolean impliedP;
        boolean inPre;
        boolean inTextArea;
        TextAreaDocument textAreaDocument;
        boolean inTitle;
        boolean lastWasNewline;
        boolean emptyAnchor;
        boolean midInsert;
        boolean inBody;
        HTML.Tag insertTag;
        boolean insertInsertTag;
        boolean foundInsertTag;
        int insertTagDepthDelta;
        int popDepth;
        int pushDepth;
        Map lastMap;
        boolean inStyle;
        String defaultStyle;
        Vector<Object> styles;
        boolean inHead;
        boolean isStyleCSS;
        boolean emptyDocument;
        AttributeSet styleAttributes;
        Option option;
        protected Vector<DefaultStyledDocument.ElementSpec> parseBuffer;
        protected MutableAttributeSet charAttr;
        Stack<AttributeSet> charAttrStack;
        Hashtable<HTML.Tag, HTMLDocument.HTMLReader.TagAction> tagMap;
        int inBlock;
        private HTML.Tag nextTagAfterPImplied;
        
        public HTMLReader(HTMLDocument htmlDocument, int n) {
            this(htmlDocument, n, 0, 0, null);
        }
        
        public HTMLReader(HTMLDocument htmlDocument, int n, int n2, int n3, HTML.Tag tag) {
            this(htmlDocument, n, n2, n3, tag, true, false, true);
        }
        
        HTMLReader(int offset, int n, int n2, HTML.Tag insertTag, boolean insertInsertTag, boolean b, boolean wantsTrailingNewline) {
            this.inParagraph = false;
            this.impliedP = false;
            this.inPre = false;
            this.inTextArea = false;
            this.textAreaDocument = null;
            this.inTitle = false;
            this.lastWasNewline = true;
            this.inStyle = false;
            this.inHead = false;
            this.parseBuffer = new Vector();
            this.charAttr = new HTMLDocument.TaggedAttributeSet();
            this.charAttrStack = new Stack();
            this.inBlock = 0;
            this.nextTagAfterPImplied = null;
            this.emptyDocument = (HTMLDocument.this.getLength() == 0);
            this.isStyleCSS = "text/css".equals(HTMLDocument.this.getDefaultStyleSheetType());
            this.offset = offset;
            this.threshold = HTMLDocument.this.getTokenThreshold();
            this.tagMap = new Hashtable(57);
            HTMLDocument.HTMLReader.TagAction tagAction = new HTMLDocument.HTMLReader.TagAction();
            HTMLDocument.HTMLReader.BlockAction value = new HTMLDocument.HTMLReader.BlockAction();
            HTMLDocument.HTMLReader.ParagraphAction paragraphAction = new HTMLDocument.HTMLReader.ParagraphAction();
            HTMLDocument.HTMLReader.CharacterAction characterAction = new HTMLDocument.HTMLReader.CharacterAction();
            HTMLDocument.HTMLReader.SpecialAction value2 = new HTMLDocument.HTMLReader.SpecialAction();
            HTMLDocument.HTMLReader.FormAction formAction = new HTMLDocument.HTMLReader.FormAction();
            HTMLDocument.HTMLReader.HiddenAction hiddenAction = new HTMLDocument.HTMLReader.HiddenAction();
            HTMLDocument.HTMLReader.ConvertAction value3 = new HTMLDocument.HTMLReader.ConvertAction();
            this.tagMap.put(HTML.Tag.A, new HTMLDocument.HTMLReader.AnchorAction());
            this.tagMap.put(HTML.Tag.ADDRESS, characterAction);
            this.tagMap.put(HTML.Tag.APPLET, hiddenAction);
            this.tagMap.put(HTML.Tag.AREA, new HTMLDocument.HTMLReader.AreaAction());
            this.tagMap.put(HTML.Tag.B, value3);
            this.tagMap.put(HTML.Tag.BASE, new HTMLDocument.HTMLReader.BaseAction());
            this.tagMap.put(HTML.Tag.BASEFONT, characterAction);
            this.tagMap.put(HTML.Tag.BIG, characterAction);
            this.tagMap.put(HTML.Tag.BLOCKQUOTE, value);
            this.tagMap.put(HTML.Tag.BODY, value);
            this.tagMap.put(HTML.Tag.BR, value2);
            this.tagMap.put(HTML.Tag.CAPTION, value);
            this.tagMap.put(HTML.Tag.CENTER, value);
            this.tagMap.put(HTML.Tag.CITE, characterAction);
            this.tagMap.put(HTML.Tag.CODE, characterAction);
            this.tagMap.put(HTML.Tag.DD, value);
            this.tagMap.put(HTML.Tag.DFN, characterAction);
            this.tagMap.put(HTML.Tag.DIR, value);
            this.tagMap.put(HTML.Tag.DIV, value);
            this.tagMap.put(HTML.Tag.DL, value);
            this.tagMap.put(HTML.Tag.DT, paragraphAction);
            this.tagMap.put(HTML.Tag.EM, characterAction);
            this.tagMap.put(HTML.Tag.FONT, value3);
            this.tagMap.put(HTML.Tag.FORM, new HTMLDocument.HTMLReader.FormTagAction());
            this.tagMap.put(HTML.Tag.FRAME, value2);
            this.tagMap.put(HTML.Tag.FRAMESET, value);
            this.tagMap.put(HTML.Tag.H1, paragraphAction);
            this.tagMap.put(HTML.Tag.H2, paragraphAction);
            this.tagMap.put(HTML.Tag.H3, paragraphAction);
            this.tagMap.put(HTML.Tag.H4, paragraphAction);
            this.tagMap.put(HTML.Tag.H5, paragraphAction);
            this.tagMap.put(HTML.Tag.H6, paragraphAction);
            this.tagMap.put(HTML.Tag.HEAD, new HTMLDocument.HTMLReader.HeadAction());
            this.tagMap.put(HTML.Tag.HR, value2);
            this.tagMap.put(HTML.Tag.HTML, value);
            this.tagMap.put(HTML.Tag.I, value3);
            this.tagMap.put(HTML.Tag.IMG, value2);
            this.tagMap.put(HTML.Tag.INPUT, formAction);
            this.tagMap.put(HTML.Tag.ISINDEX, new HTMLDocument.HTMLReader.IsindexAction());
            this.tagMap.put(HTML.Tag.KBD, characterAction);
            this.tagMap.put(HTML.Tag.LI, value);
            this.tagMap.put(HTML.Tag.LINK, new HTMLDocument.HTMLReader.LinkAction());
            this.tagMap.put(HTML.Tag.MAP, new HTMLDocument.HTMLReader.MapAction());
            this.tagMap.put(HTML.Tag.MENU, value);
            this.tagMap.put(HTML.Tag.META, new HTMLDocument.HTMLReader.MetaAction());
            this.tagMap.put(HTML.Tag.NOBR, characterAction);
            this.tagMap.put(HTML.Tag.NOFRAMES, value);
            this.tagMap.put(HTML.Tag.OBJECT, value2);
            this.tagMap.put(HTML.Tag.OL, value);
            this.tagMap.put(HTML.Tag.OPTION, formAction);
            this.tagMap.put(HTML.Tag.P, paragraphAction);
            this.tagMap.put(HTML.Tag.PARAM, new HTMLDocument.HTMLReader.ObjectAction());
            this.tagMap.put(HTML.Tag.PRE, new HTMLDocument.HTMLReader.PreAction());
            this.tagMap.put(HTML.Tag.SAMP, characterAction);
            this.tagMap.put(HTML.Tag.SCRIPT, hiddenAction);
            this.tagMap.put(HTML.Tag.SELECT, formAction);
            this.tagMap.put(HTML.Tag.SMALL, characterAction);
            this.tagMap.put(HTML.Tag.SPAN, characterAction);
            this.tagMap.put(HTML.Tag.STRIKE, value3);
            this.tagMap.put(HTML.Tag.S, characterAction);
            this.tagMap.put(HTML.Tag.STRONG, characterAction);
            this.tagMap.put(HTML.Tag.STYLE, new HTMLDocument.HTMLReader.StyleAction());
            this.tagMap.put(HTML.Tag.SUB, value3);
            this.tagMap.put(HTML.Tag.SUP, value3);
            this.tagMap.put(HTML.Tag.TABLE, value);
            this.tagMap.put(HTML.Tag.TD, value);
            this.tagMap.put(HTML.Tag.TEXTAREA, formAction);
            this.tagMap.put(HTML.Tag.TH, value);
            this.tagMap.put(HTML.Tag.TITLE, new HTMLDocument.HTMLReader.TitleAction());
            this.tagMap.put(HTML.Tag.TR, value);
            this.tagMap.put(HTML.Tag.TT, characterAction);
            this.tagMap.put(HTML.Tag.U, value3);
            this.tagMap.put(HTML.Tag.UL, value);
            this.tagMap.put(HTML.Tag.VAR, characterAction);
            if (insertTag != null) {
                this.insertTag = insertTag;
                this.popDepth = n;
                this.pushDepth = n2;
                this.insertInsertTag = insertInsertTag;
                this.foundInsertTag = false;
            }
            else {
                this.foundInsertTag = true;
            }
            if (b) {
                this.popDepth = n;
                this.pushDepth = n2;
                this.insertAfterImplied = true;
                this.foundInsertTag = false;
                this.midInsert = false;
                this.insertInsertTag = true;
                this.wantsTrailingNewline = wantsTrailingNewline;
            }
            else {
                this.midInsert = (!this.emptyDocument && insertTag == null);
                if (this.midInsert) {
                    this.generateEndsSpecsForMidInsert();
                }
            }
            if (!this.emptyDocument && !this.midInsert) {
                Element element = HTMLDocument.this.getCharacterElement(Math.max(this.offset - 1, 0));
                for (int i = 0; i <= this.popDepth; ++i) {
                    element = element.getParentElement();
                }
                for (int j = 0; j < this.pushDepth; ++j) {
                    element = element.getElement(element.getElementIndex(this.offset));
                }
                AttributeSet attributes = element.getAttributes();
                if (attributes != null) {
                    HTML.Tag tag = (HTML.Tag)attributes.getAttribute(StyleConstants.NameAttribute);
                    if (tag != null) {
                        this.inParagraph = tag.isParagraph();
                    }
                }
            }
        }
        
        private void generateEndsSpecsForMidInsert() {
            int n = this.heightToElementWithName(HTML.Tag.BODY, Math.max(0, this.offset - 1));
            boolean n2 = false;
            if (n == -1 && this.offset > 0) {
                n = this.heightToElementWithName(HTML.Tag.BODY, this.offset);
                if (n != -1) {
                    n = this.depthTo(this.offset - 1) - 1;
                    n2 = true;
                }
            }
            if (n == -1) {
                throw new RuntimeException("Must insert new content into body element-");
            }
            if (n != -1) {
                try {
                    if (n2 == 0 && this.offset > 0 && !HTMLDocument.this.getText(this.offset - 1, 1).equals("\n")) {
                        SimpleAttributeSet a = new SimpleAttributeSet();
                        a.addAttribute(StyleConstants.NameAttribute, HTML.Tag.CONTENT);
                        this.parseBuffer.addElement(new DefaultStyledDocument.ElementSpec(a, (short)3, HTMLDocument.NEWLINE, 0, 1));
                    }
                }
                catch (BadLocationException ex) {}
                while (n-- > 0) {
                    this.parseBuffer.addElement(new DefaultStyledDocument.ElementSpec(null, (short)2));
                }
                if (!(n2 == 0)) {
                    DefaultStyledDocument.ElementSpec obj = new DefaultStyledDocument.ElementSpec(null, (short)1);
                    obj.setDirection((short)5);
                    this.parseBuffer.addElement(obj);
                }
            }
            return;
        }
        
        private int depthTo(int n) {
            Element element = HTMLDocument.this.getDefaultRootElement();
            int n2 = 0;
            while (!element.isLeaf()) {
                ++n2;
                element = element.getElement(element.getElementIndex(n));
            }
            return n2;
        }
        
        private int heightToElementWithName(Object o, int pos) {
            Element element = HTMLDocument.this.getCharacterElement(pos).getParentElement();
            int n = 0;
            while (element != null && element.getAttributes().getAttribute(StyleConstants.NameAttribute) != o) {
                ++n;
                element = element.getParentElement();
            }
            return (element == null) ? -1 : n;
        }
        
        private void adjustEndElement() {
            int length = HTMLDocument.this.getLength();
            if (length == 0) {
                return;
            }
            HTMLDocument.this.obtainLock();
            try {
                Element[] pathTo = this.getPathTo(length - 1);
                int length2 = pathTo.length;
                if (length2 > 1 && pathTo[1].getAttributes().getAttribute(StyleConstants.NameAttribute) == HTML.Tag.BODY && pathTo[1].getEndOffset() == length) {
                    String text = HTMLDocument.this.getText(length - 1, 1);
                    Element[] array = new Element[0];
                    Element[] removed = { null };
                    int elementIndex = pathTo[0].getElementIndex(length);
                    removed[0] = pathTo[0].getElement(elementIndex);
                    ((AbstractDocument.BranchElement)pathTo[0]).replace(elementIndex, 1, array);
                    AbstractDocument.ElementEdit anEdit = new AbstractDocument.ElementEdit(pathTo[0], elementIndex, removed, array);
                    SimpleAttributeSet set = new SimpleAttributeSet();
                    set.addAttribute(StyleConstants.NameAttribute, HTML.Tag.CONTENT);
                    set.addAttribute("CR", Boolean.TRUE);
                    Element[] array2 = { HTMLDocument.this.createLeafElement(pathTo[length2 - 1], set, length, length + 1) };
                    int elementCount = pathTo[length2 - 1].getElementCount();
                    ((AbstractDocument.BranchElement)pathTo[length2 - 1]).replace(elementCount, 0, array2);
                    AbstractDocument.DefaultDocumentEvent edit = new AbstractDocument.DefaultDocumentEvent(length, 1, DocumentEvent.EventType.CHANGE);
                    edit.addEdit(new AbstractDocument.ElementEdit(pathTo[length2 - 1], elementCount, new Element[0], array2));
                    edit.addEdit(anEdit);
                    edit.end();
                    HTMLDocument.this.fireChangedUpdate(edit);
                    HTMLDocument.this.fireUndoableEditUpdate(new UndoableEditEvent(this, edit));
                    if (text.equals("\n")) {
                        AbstractDocument.DefaultDocumentEvent edit2 = new AbstractDocument.DefaultDocumentEvent(length - 1, 1, DocumentEvent.EventType.REMOVE);
                        HTMLDocument.access$200(HTMLDocument.this, edit2);
                        UndoableEdit remove = HTMLDocument.access$300(HTMLDocument.this).remove(length - 1, 1);
                        if (remove != null) {
                            edit2.addEdit(remove);
                        }
                        HTMLDocument.access$400(HTMLDocument.this, edit2);
                        edit2.end();
                        HTMLDocument.access$500(HTMLDocument.this, edit2);
                        HTMLDocument.this.fireUndoableEditUpdate(new UndoableEditEvent(this, edit2));
                    }
                }
            }
            catch (BadLocationException ex) {}
            finally {
                HTMLDocument.this.releaseLock();
            }
            return;
        }
        
        private Element[] getPathTo(int n) {
            Stack<Element> stack = new Stack();
            for (Element item = HTMLDocument.this.getDefaultRootElement(); !item.isLeaf(); item = item.getElement(item.getElementIndex(n))) {
                stack.push(item);
            }
            Element[] anArray = new Element[stack.size()];
            stack.copyInto(anArray);
            return anArray;
        }
        
        @Override
        public void flush() throws BadLocationException {
            if (this.emptyDocument && !this.insertAfterImplied) {
                if (HTMLDocument.this.getLength() > 0 || this.parseBuffer.size() > 0) {
                    this.flushBuffer(true);
                    this.adjustEndElement();
                }
            }
            else {
                this.flushBuffer(true);
            }
        }
        
        @Override
        public void handleText(char[] array, int n) {
            if (this.receivedEndHTML || (this.midInsert && !this.inBody)) {
                return;
            }
            if (HTMLDocument.this.getProperty("i18n").equals(Boolean.FALSE)) {
                Object property = HTMLDocument.this.getProperty(TextAttribute.RUN_DIRECTION);
                if (property != null && property.equals(TextAttribute.RUN_DIRECTION_RTL)) {
                    HTMLDocument.this.putProperty("i18n", Boolean.TRUE);
                }
                else if (SwingUtilities2.isComplexLayout(array, 0, array.length)) {
                    HTMLDocument.this.putProperty("i18n", Boolean.TRUE);
                }
            }
            if (this.inTextArea) {
                this.textAreaContent(array);
            }
            else if (this.inPre) {
                this.preContent(array);
            }
            else if (this.inTitle) {
                HTMLDocument.this.putProperty("title", new String(array));
            }
            else if (this.option != null) {
                this.option.setLabel(new String(array));
            }
            else if (this.inStyle) {
                if (this.styles != null) {
                    this.styles.addElement(new String(array));
                }
            }
            else if (this.inBlock > 0) {
                if (!this.foundInsertTag && this.insertAfterImplied) {
                    this.foundInsertTag(false);
                    this.foundInsertTag = true;
                    boolean b = !HTMLDocument.this.insertInBody;
                    this.impliedP = b;
                    this.inParagraph = b;
                }
                if (array.length >= 1) {
                    this.addContent(array, 0, array.length);
                }
            }
            return;
        }
        
        @Override
        public void handleStartTag(HTML.Tag key, MutableAttributeSet set, int n) {
            if (this.receivedEndHTML) {
                return;
            }
            if (this.midInsert && !this.inBody) {
                if (key == HTML.Tag.BODY) {
                    this.inBody = true;
                    ++this.inBlock;
                }
                return;
            }
            else {
                if (!this.inBody && key == HTML.Tag.BODY) {
                    this.inBody = true;
                }
                if (this.isStyleCSS && set.isDefined(HTML.Attribute.STYLE)) {
                    String s = (String)set.getAttribute(HTML.Attribute.STYLE);
                    set.removeAttribute(HTML.Attribute.STYLE);
                    set.addAttributes(this.styleAttributes = HTMLDocument.this.getStyleSheet().getDeclaration(s));
                }
                else {
                    this.styleAttributes = null;
                }
                HTMLDocument.HTMLReader.TagAction tagAction = (HTMLDocument.HTMLReader.TagAction)this.tagMap.get(key);
                if (tagAction != null) {
                    tagAction.start(key, set);
                }
                return;
            }
        }
        
        @Override
        public void handleComment(char[] array, int n) {
            if (this.receivedEndHTML) {
                this.addExternalComment(new String(array));
                return;
            }
            if (this.inStyle) {
                if (this.styles != null) {
                    this.styles.addElement(new String(array));
                }
            }
            else if (HTMLDocument.this.getPreservesUnknownTags()) {
                if (this.inBlock == 0 && (this.foundInsertTag || this.insertTag != HTML.Tag.COMMENT)) {
                    this.addExternalComment(new String(array));
                    return;
                }
                SimpleAttributeSet set = new SimpleAttributeSet();
                set.addAttribute(HTML.Attribute.COMMENT, new String(array));
                this.addSpecialElement(HTML.Tag.COMMENT, set);
            }
            HTMLDocument.HTMLReader.TagAction tagAction = (HTMLDocument.HTMLReader.TagAction)this.tagMap.get(HTML.Tag.COMMENT);
            if (tagAction != null) {
                tagAction.start(HTML.Tag.COMMENT, new SimpleAttributeSet());
                tagAction.end(HTML.Tag.COMMENT);
            }
            return;
        }
        
        private void addExternalComment(String obj) {
            Object property = HTMLDocument.this.getProperty("AdditionalComments");
            if (property != null && !(property instanceof Vector)) {
                return;
            }
            if (property == null) {
                property = new Vector();
                HTMLDocument.this.putProperty("AdditionalComments", property);
            }
            ((Vector)property).addElement(obj);
            return;
        }
        
        @Override
        public void handleEndTag(HTML.Tag key, int n) {
            if (this.receivedEndHTML || (this.midInsert && !this.inBody)) {
                return;
            }
            if (key == HTML.Tag.HTML) {
                this.receivedEndHTML = true;
            }
            if (key == HTML.Tag.BODY) {
                this.inBody = false;
                if (this.midInsert) {
                    --this.inBlock;
                }
            }
            HTMLDocument.HTMLReader.TagAction tagAction = (HTMLDocument.HTMLReader.TagAction)this.tagMap.get(key);
            if (tagAction != null) {
                tagAction.end(key);
            }
            return;
        }
        
        @Override
        public void handleSimpleTag(HTML.Tag key, MutableAttributeSet set, int n) {
            if (this.receivedEndHTML || (this.midInsert && !this.inBody)) {
                return;
            }
            if (this.isStyleCSS && set.isDefined(HTML.Attribute.STYLE)) {
                String s = (String)set.getAttribute(HTML.Attribute.STYLE);
                set.removeAttribute(HTML.Attribute.STYLE);
                set.addAttributes(this.styleAttributes = HTMLDocument.this.getStyleSheet().getDeclaration(s));
            }
            else {
                this.styleAttributes = null;
            }
            HTMLDocument.HTMLReader.TagAction tagAction = (HTMLDocument.HTMLReader.TagAction)this.tagMap.get(key);
            if (tagAction != null) {
                tagAction.start(key, set);
                tagAction.end(key);
            }
            else if (HTMLDocument.this.getPreservesUnknownTags()) {
                this.addSpecialElement(key, set);
            }
            return;
        }
        
        @Override
        public void handleEndOfLineString(String value) {
            if (this.emptyDocument && value != null) {
                HTMLDocument.this.putProperty("__EndOfLine__", value);
            }
        }
        
        protected void registerTag(HTML.Tag key, HTMLDocument.HTMLReader.TagAction value) {
            this.tagMap.put(key, value);
        }
        
        protected void pushCharacterStyle() {
            this.charAttrStack.push(this.charAttr.copyAttributes());
        }
        
        protected void popCharacterStyle() {
            if (!this.charAttrStack.empty()) {
                this.charAttr = (MutableAttributeSet)this.charAttrStack.peek();
                this.charAttrStack.pop();
            }
        }
        
        protected void textAreaContent(char[] value) {
            try {
                this.textAreaDocument.insertString(this.textAreaDocument.getLength(), new String(value), null);
            }
            catch (BadLocationException ex) {}
        }
        
        protected void preContent(char[] array) {
            int n = 0;
            for (int i = 0; i < array.length; ++i) {
                if (array[i] == '\n') {
                    this.addContent(array, n, i - n + 1);
                    this.blockClose(HTML.Tag.IMPLIED);
                    SimpleAttributeSet o = new SimpleAttributeSet();
                    o.addAttribute(CSS.Attribute.WHITE_SPACE, "pre");
                    this.blockOpen(HTML.Tag.IMPLIED, o);
                    n = i + 1;
                }
            }
            if (n < array.length) {
                this.addContent(array, n, array.length - n);
            }
        }
        
        protected void blockOpen(HTML.Tag tag, MutableAttributeSet set) {
            if (this.impliedP) {
                this.blockClose(HTML.Tag.IMPLIED);
            }
            ++this.inBlock;
            if (!this.canInsertTag(tag, set, true)) {
                return;
            }
            if (set.isDefined(HTMLDocument.HTMLReader.IMPLIED)) {
                set.removeAttribute(HTMLDocument.HTMLReader.IMPLIED);
            }
            this.lastWasNewline = false;
            set.addAttribute(StyleConstants.NameAttribute, tag);
            this.parseBuffer.addElement(new DefaultStyledDocument.ElementSpec(set.copyAttributes(), (short)1));
            return;
        }
        
        protected void blockClose(HTML.Tag tag) {
            --this.inBlock;
            if (!this.foundInsertTag) {
                return;
            }
            if (!this.lastWasNewline) {
                this.pushCharacterStyle();
                this.charAttr.addAttribute("CR", Boolean.TRUE);
                this.addContent(HTMLDocument.NEWLINE, 0, 1, true);
                this.popCharacterStyle();
                this.lastWasNewline = true;
            }
            if (this.impliedP) {
                this.impliedP = false;
                this.inParagraph = false;
                if (tag != HTML.Tag.IMPLIED) {
                    this.blockClose(HTML.Tag.IMPLIED);
                }
            }
            DefaultStyledDocument.ElementSpec elementSpec = (this.parseBuffer.size() > 0) ? ((DefaultStyledDocument.ElementSpec)this.parseBuffer.lastElement()) : null;
            if (elementSpec != null && elementSpec.getType() == 1) {
                this.addContent(new char[] { ' ' }, 0, 1);
            }
            this.parseBuffer.addElement(new DefaultStyledDocument.ElementSpec(null, (short)2));
            return;
        }
        
        protected void addContent(char[] array, int n, int n2) {
            this.addContent(array, n, n2, true);
        }
        
        protected void addContent(char[] txt, int offs, int len, boolean b) {
            if (!this.foundInsertTag) {
                return;
            }
            if (b && !this.inParagraph && !this.inPre) {
                this.blockOpen(HTML.Tag.IMPLIED, new SimpleAttributeSet());
                this.inParagraph = true;
                this.impliedP = true;
            }
            this.emptyAnchor = false;
            this.charAttr.addAttribute(StyleConstants.NameAttribute, HTML.Tag.CONTENT);
            this.parseBuffer.addElement(new DefaultStyledDocument.ElementSpec(this.charAttr.copyAttributes(), (short)3, txt, offs, len));
            if (this.parseBuffer.size() > this.threshold) {
                if (this.threshold <= 10000) {
                    this.threshold *= 5;
                }
                try {
                    this.flushBuffer(false);
                }
                catch (BadLocationException ex) {}
            }
            if (len > 0) {
                this.lastWasNewline = (txt[offs + len - 1] == '\n');
            }
            return;
        }
        
        protected void addSpecialElement(HTML.Tag nextTagAfterPImplied, MutableAttributeSet set) {
            if (nextTagAfterPImplied != HTML.Tag.FRAME && !this.inParagraph && !this.inPre) {
                this.nextTagAfterPImplied = nextTagAfterPImplied;
                this.blockOpen(HTML.Tag.IMPLIED, new SimpleAttributeSet());
                this.nextTagAfterPImplied = null;
                this.inParagraph = true;
                this.impliedP = true;
            }
            if (!this.canInsertTag(nextTagAfterPImplied, set, nextTagAfterPImplied.isBlock())) {
                return;
            }
            if (set.isDefined(HTMLDocument.HTMLReader.IMPLIED)) {
                set.removeAttribute(HTMLDocument.HTMLReader.IMPLIED);
            }
            this.emptyAnchor = false;
            set.addAttributes(this.charAttr);
            set.addAttribute(StyleConstants.NameAttribute, nextTagAfterPImplied);
            this.parseBuffer.addElement(new DefaultStyledDocument.ElementSpec(set.copyAttributes(), (short)3, new char[] { ' ' }, 0, 1));
            if (nextTagAfterPImplied == HTML.Tag.FRAME) {
                this.lastWasNewline = true;
            }
            return;
        }
        
        void flushBuffer(boolean b) throws BadLocationException {
            int length = HTMLDocument.this.getLength();
            int n = this.parseBuffer.size();
            if (b && (this.insertTag != null || this.insertAfterImplied) && n > 0) {
                this.adjustEndSpecsForPartialInsert();
                n = this.parseBuffer.size();
            }
            DefaultStyledDocument.ElementSpec[] anArray = new DefaultStyledDocument.ElementSpec[n];
            this.parseBuffer.copyInto(anArray);
            if (length == 0 && this.insertTag == null && !this.insertAfterImplied) {
                HTMLDocument.this.create(anArray);
            }
            else {
                HTMLDocument.this.insert(this.offset, anArray);
            }
            this.parseBuffer.removeAllElements();
            this.offset += HTMLDocument.this.getLength() - length;
            ++this.flushCount;
        }
        
        private void adjustEndSpecsForPartialInsert() {
            int i = this.parseBuffer.size();
            if (this.insertTagDepthDelta < 0) {
                for (int insertTagDepthDelta = this.insertTagDepthDelta; insertTagDepthDelta < 0 && i >= 0 && ((DefaultStyledDocument.ElementSpec)this.parseBuffer.elementAt(i - 1)).getType() == 2; ++insertTagDepthDelta) {
                    this.parseBuffer.removeElementAt(--i);
                }
            }
            if (this.flushCount == 0 && (!this.insertAfterImplied || !this.wantsTrailingNewline)) {
                int n = 0;
                if (this.pushDepth > 0 && ((DefaultStyledDocument.ElementSpec)this.parseBuffer.elementAt(0)).getType() == 3) {
                    ++n;
                }
                int n2;
                int n3;
                for (n2 = n + (this.popDepth + this.pushDepth), n3 = 0, ; n2 < i && ((DefaultStyledDocument.ElementSpec)this.parseBuffer.elementAt(n2)).getType() == 3; ++n2, ++n3) {}
                if (n3 > 1) {
                    while (n2 < i && ((DefaultStyledDocument.ElementSpec)this.parseBuffer.elementAt(n2)).getType() == 2) {
                        ++n2;
                    }
                    if (n2 == i) {
                        char[] array = ((DefaultStyledDocument.ElementSpec)this.parseBuffer.elementAt(n2 + n3 - 1)).getArray();
                        if (array.length == 1 && array[0] == HTMLDocument.NEWLINE[0]) {
                            while (i > n2 + n3 - 1) {
                                this.parseBuffer.removeElementAt(--i);
                            }
                        }
                    }
                }
            }
            if (this.wantsTrailingNewline) {
                int j = this.parseBuffer.size() - 1;
                while (j >= 0) {
                    DefaultStyledDocument.ElementSpec elementSpec = (DefaultStyledDocument.ElementSpec)this.parseBuffer.elementAt(j);
                    if (elementSpec.getType() == 3) {
                        if (elementSpec.getArray()[elementSpec.getLength() - 1] != '\n') {
                            SimpleAttributeSet a = new SimpleAttributeSet();
                            a.addAttribute(StyleConstants.NameAttribute, HTML.Tag.CONTENT);
                            this.parseBuffer.insertElementAt(new DefaultStyledDocument.ElementSpec(a, (short)3, HTMLDocument.NEWLINE, 0, 1), j + 1);
                            break;
                        }
                        break;
                    }
                    else {
                        --j;
                    }
                }
            }
        }
        
        void addCSSRules(String s) {
            HTMLDocument.this.getStyleSheet().addRule(s);
        }
        
        void linkCSSStyleSheet(String s) {
            URL url;
            try {
                url = new URL(HTMLDocument.this.base, s);
            }
            catch (MalformedURLException ex) {
                try {
                    url = new URL(s);
                }
                catch (MalformedURLException ex2) {
                    url = null;
                }
            }
            if (url != null) {
                HTMLDocument.this.getStyleSheet().importStyleSheet(url);
            }
        }
        
        private boolean canInsertTag(HTML.Tag tag, AttributeSet set, boolean b) {
            if (!this.foundInsertTag) {
                if (tag == HTML.Tag.IMPLIED && !this.inParagraph && !this.inPre && this.nextTagAfterPImplied != null) {
                    if (this.insertTag != null && (!this.isInsertTag(this.nextTagAfterPImplied) || !this.insertInsertTag)) {
                        return false;
                    }
                }
                else if ((this.insertTag != null && !this.isInsertTag(tag)) || (this.insertAfterImplied && (set == null || set.isDefined(HTMLDocument.HTMLReader.IMPLIED) || tag == HTML.Tag.IMPLIED))) {
                    return false;
                }
                this.foundInsertTag(b);
                if (!this.insertInsertTag) {
                    return false;
                }
            }
            return true;
        }
        
        private boolean isInsertTag(HTML.Tag tag) {
            return this.insertTag == tag;
        }
        
        private void foundInsertTag(boolean b) {
            this.foundInsertTag = true;
            Label_0236: {
                if (!this.insertAfterImplied) {
                    if (this.popDepth <= 0 && this.pushDepth <= 0) {
                        break Label_0236;
                    }
                    try {
                        if (this.offset == 0 || !getText(this.offset - 1, 1).equals("\n")) {
                            Object a = null;
                            boolean n = true;
                            if (this.offset != 0) {
                                AttributeSet attributes = getCharacterElement(this.offset - 1).getAttributes();
                                if (attributes.isDefined(StyleConstants.ComposedTextAttribute)) {
                                    n = false;
                                }
                                else {
                                    Object attribute = attributes.getAttribute(StyleConstants.NameAttribute);
                                    if ((attribute instanceof HTML.Tag)) {
                                        HTML.Tag tag = (HTML.Tag)attribute;
                                        if (tag == HTML.Tag.IMG || tag == HTML.Tag.HR || tag == HTML.Tag.COMMENT || tag instanceof HTML.UnknownTag) {
                                            n = false;
                                        }
                                    }
                                }
                            }
                            if (n == 0) {
                                a = new SimpleAttributeSet();
                                ((SimpleAttributeSet)a).addAttribute(StyleConstants.NameAttribute, HTML.Tag.CONTENT);
                            }
                            DefaultStyledDocument.ElementSpec obj = new DefaultStyledDocument.ElementSpec(a, (short)3, HTMLDocument.NEWLINE, 0, HTMLDocument.NEWLINE.length);
                            if (!(n == 0)) {
                                obj.setDirection((short)4);
                            }
                            this.parseBuffer.addElement(obj);
                        }
                    }
                    catch (BadLocationException ex) {}
                }
            }
            for (int i = 0; i < this.popDepth; ++i) {
                this.parseBuffer.addElement(new DefaultStyledDocument.ElementSpec(null, (short)2));
            }
            for (int j = 0; j < this.pushDepth; ++j) {
                DefaultStyledDocument.ElementSpec obj2 = new DefaultStyledDocument.ElementSpec(null, (short)1);
                obj2.setDirection((short)5);
                this.parseBuffer.addElement(obj2);
            }
            this.insertTagDepthDelta = this.depthTo(Math.max(0, this.offset - 1)) - this.popDepth + this.pushDepth - this.inBlock;
            if (b) {
                ++this.insertTagDepthDelta;
            }
            else {
                --this.insertTagDepthDelta;
                this.inParagraph = true;
                this.lastWasNewline = false;
            }
        }
        
        static boolean access$900(HTMLDocument.HTMLReader htmlReader, HTML.Tag tag, AttributeSet set, boolean b) {
            return htmlReader.canInsertTag(tag, set, b);
        }
        
        public class TagAction
        {
            public void start(HTML.Tag tag, MutableAttributeSet set) {
            }
            
            public void end(HTML.Tag tag) {
            }
        }
        
        public class BlockAction extends HTMLDocument.HTMLReader.TagAction
        {
            @Override
            public void start(HTML.Tag tag, MutableAttributeSet set) {
                HTMLDocument.HTMLReader.this.blockOpen(tag, set);
            }
            
            @Override
            public void end(HTML.Tag tag) {
                HTMLDocument.HTMLReader.this.blockClose(tag);
            }
        }
        
        private class FormTagAction extends HTMLDocument.HTMLReader.BlockAction
        {
            @Override
            public void start(HTML.Tag tag, MutableAttributeSet set) {
                super.start(tag, set);
                HTMLDocument.this.radioButtonGroupsMap = new HashMap();
            }
            
            @Override
            public void end(HTML.Tag tag) {
                super.end(tag);
                HTMLDocument.this.radioButtonGroupsMap = null;
            }
        }
        
        public class ParagraphAction extends HTMLDocument.HTMLReader.BlockAction
        {
            @Override
            public void start(HTML.Tag tag, MutableAttributeSet set) {
                super.start(tag, set);
                HTMLDocument.HTMLReader.this.inParagraph = true;
            }
            
            @Override
            public void end(HTML.Tag tag) {
                super.end(tag);
                HTMLDocument.HTMLReader.this.inParagraph = false;
            }
        }
        
        public class SpecialAction extends HTMLDocument.HTMLReader.TagAction
        {
            @Override
            public void start(HTML.Tag tag, MutableAttributeSet set) {
                HTMLDocument.HTMLReader.this.addSpecialElement(tag, set);
            }
        }
        
        public class IsindexAction extends HTMLDocument.HTMLReader.TagAction
        {
            @Override
            public void start(HTML.Tag tag, MutableAttributeSet set) {
                HTMLDocument.HTMLReader.this.blockOpen(HTML.Tag.IMPLIED, new SimpleAttributeSet());
                HTMLDocument.HTMLReader.this.addSpecialElement(tag, set);
                HTMLDocument.HTMLReader.this.blockClose(HTML.Tag.IMPLIED);
            }
        }
        
        public class HiddenAction extends HTMLDocument.HTMLReader.TagAction
        {
            @Override
            public void start(HTML.Tag tag, MutableAttributeSet set) {
                HTMLDocument.HTMLReader.this.addSpecialElement(tag, set);
            }
            
            @Override
            public void end(HTML.Tag tag) {
                if (!this.isEmpty(tag)) {
                    SimpleAttributeSet set = new SimpleAttributeSet();
                    set.addAttribute(HTML.Attribute.ENDTAG, "true");
                    HTMLDocument.HTMLReader.this.addSpecialElement(tag, set);
                }
            }
            
            boolean isEmpty(HTML.Tag tag) {
                return tag != HTML.Tag.APPLET && tag != HTML.Tag.SCRIPT;
            }
        }
        
        class MetaAction extends HTMLDocument.HTMLReader.HiddenAction
        {
            @Override
            public void start(HTML.Tag tag, MutableAttributeSet set) {
                Object attribute = set.getAttribute(HTML.Attribute.HTTPEQUIV);
                if (attribute != null) {
                    String lowerCase = ((String)attribute).toLowerCase();
                    if (lowerCase.equals("content-style-type")) {
                        HTMLDocument.this.setDefaultStyleSheetType((String)set.getAttribute(HTML.Attribute.CONTENT));
                        HTMLDocument.HTMLReader.this.isStyleCSS = "text/css".equals(HTMLDocument.this.getDefaultStyleSheetType());
                    }
                    else if (lowerCase.equals("default-style")) {
                        HTMLDocument.HTMLReader.this.defaultStyle = (String)set.getAttribute(HTML.Attribute.CONTENT);
                    }
                }
                super.start(tag, set);
            }
            
            @Override
            boolean isEmpty(HTML.Tag tag) {
                return true;
            }
        }
        
        class HeadAction extends HTMLDocument.HTMLReader.BlockAction
        {
            @Override
            public void start(HTML.Tag tag, MutableAttributeSet set) {
                HTMLDocument.HTMLReader.this.inHead = true;
                if ((HTMLDocument.HTMLReader.this.insertTag == null && !HTMLDocument.HTMLReader.this.insertAfterImplied) || HTMLDocument.HTMLReader.this.insertTag == HTML.Tag.HEAD || (HTMLDocument.HTMLReader.this.insertAfterImplied && (HTMLDocument.HTMLReader.this.foundInsertTag || !set.isDefined(HTMLEditorKit.ParserCallback.IMPLIED)))) {
                    super.start(tag, set);
                }
            }
            
            @Override
            public void end(HTML.Tag tag) {
                HTMLDocument.HTMLReader this$1 = HTMLDocument.HTMLReader.this;
                HTMLDocument.HTMLReader this$2 = HTMLDocument.HTMLReader.this;
                boolean b = false;
                this$2.inStyle = b;
                this$1.inHead = b;
                if (HTMLDocument.HTMLReader.this.styles != null) {
                    int isStyleCSS = HTMLDocument.HTMLReader.this.isStyleCSS;
                    for (int i = 0, size = HTMLDocument.HTMLReader.this.styles.size(); i < size; ++i) {
                        if (HTMLDocument.HTMLReader.this.styles.elementAt(i) == HTML.Tag.LINK) {
                            this.handleLink((AttributeSet)HTMLDocument.HTMLReader.this.styles.elementAt(++i));
                        }
                        else {
                            String s = (String)HTMLDocument.HTMLReader.this.styles.elementAt(++i);
                            int n = (s == null) ? isStyleCSS : s.equals("text/css");
                            while (++i < size && HTMLDocument.HTMLReader.this.styles.elementAt(i) instanceof String) {
                                if (n == 0) {
                                    continue;
                                }
                                HTMLDocument.HTMLReader.this.addCSSRules((String)HTMLDocument.HTMLReader.this.styles.elementAt(i));
                            }
                        }
                    }
                }
                if ((HTMLDocument.HTMLReader.this.insertTag == null && !HTMLDocument.HTMLReader.this.insertAfterImplied) || HTMLDocument.HTMLReader.this.insertTag == HTML.Tag.HEAD || (HTMLDocument.HTMLReader.this.insertAfterImplied && HTMLDocument.HTMLReader.this.foundInsertTag)) {
                    super.end(tag);
                }
            }
            
            boolean isEmpty(HTML.Tag tag) {
                return false;
            }
            
            private void handleLink(AttributeSet set) {
                String defaultStyleSheetType = (String)set.getAttribute(HTML.Attribute.TYPE);
                if (defaultStyleSheetType == null) {
                    defaultStyleSheetType = HTMLDocument.this.getDefaultStyleSheetType();
                }
                if (defaultStyleSheetType.equals("text/css")) {
                    String s = (String)set.getAttribute(HTML.Attribute.REL);
                    String s2 = (String)set.getAttribute(HTML.Attribute.TITLE);
                    String s3 = (String)set.getAttribute(HTML.Attribute.MEDIA);
                    String lowerCase;
                    if (s3 == null) {
                        lowerCase = "all";
                    }
                    else {
                        lowerCase = s3.toLowerCase();
                    }
                    if (s != null) {
                        String lowerCase2 = s.toLowerCase();
                        if ((lowerCase.indexOf("all") != -1 || lowerCase.indexOf("screen") != -1) && (lowerCase2.equals("stylesheet") || (lowerCase2.equals("alternate stylesheet") && s2.equals(HTMLDocument.HTMLReader.this.defaultStyle)))) {
                            HTMLDocument.HTMLReader.this.linkCSSStyleSheet((String)set.getAttribute(HTML.Attribute.HREF));
                        }
                    }
                }
            }
        }
        
        class LinkAction extends HTMLDocument.HTMLReader.HiddenAction
        {
            @Override
            public void start(HTML.Tag obj, MutableAttributeSet set) {
                String s = (String)set.getAttribute(HTML.Attribute.REL);
                if (s != null) {
                    String lowerCase = s.toLowerCase();
                    if (lowerCase.equals("stylesheet") || lowerCase.equals("alternate stylesheet")) {
                        if (HTMLDocument.HTMLReader.this.styles == null) {
                            HTMLDocument.HTMLReader.this.styles = new Vector(3);
                        }
                        HTMLDocument.HTMLReader.this.styles.addElement(obj);
                        HTMLDocument.HTMLReader.this.styles.addElement(set.copyAttributes());
                    }
                }
                super.start(obj, set);
            }
        }
        
        class MapAction extends HTMLDocument.HTMLReader.TagAction
        {
            @Override
            public void start(HTML.Tag tag, MutableAttributeSet set) {
                HTMLDocument.HTMLReader.this.lastMap = new Map((String)set.getAttribute(HTML.Attribute.NAME));
                HTMLDocument.this.addMap(HTMLDocument.HTMLReader.this.lastMap);
            }
            
            @Override
            public void end(HTML.Tag tag) {
            }
        }
        
        class AreaAction extends HTMLDocument.HTMLReader.TagAction
        {
            @Override
            public void start(HTML.Tag tag, MutableAttributeSet set) {
                if (HTMLDocument.HTMLReader.this.lastMap != null) {
                    HTMLDocument.HTMLReader.this.lastMap.addArea(set.copyAttributes());
                }
            }
            
            @Override
            public void end(HTML.Tag tag) {
            }
        }
        
        class StyleAction extends HTMLDocument.HTMLReader.TagAction
        {
            @Override
            public void start(HTML.Tag obj, MutableAttributeSet set) {
                if (HTMLDocument.HTMLReader.this.inHead) {
                    if (HTMLDocument.HTMLReader.this.styles == null) {
                        HTMLDocument.HTMLReader.this.styles = new Vector(3);
                    }
                    HTMLDocument.HTMLReader.this.styles.addElement(obj);
                    HTMLDocument.HTMLReader.this.styles.addElement(set.getAttribute(HTML.Attribute.TYPE));
                    HTMLDocument.HTMLReader.this.inStyle = true;
                }
            }
            
            @Override
            public void end(HTML.Tag tag) {
                HTMLDocument.HTMLReader.this.inStyle = false;
            }
            
            boolean isEmpty(HTML.Tag tag) {
                return false;
            }
        }
        
        public class PreAction extends HTMLDocument.HTMLReader.BlockAction
        {
            @Override
            public void start(HTML.Tag tag, MutableAttributeSet set) {
                HTMLDocument.HTMLReader.this.inPre = true;
                HTMLDocument.HTMLReader.this.blockOpen(tag, set);
                set.addAttribute(CSS.Attribute.WHITE_SPACE, "pre");
                HTMLDocument.HTMLReader.this.blockOpen(HTML.Tag.IMPLIED, set);
            }
            
            @Override
            public void end(HTML.Tag tag) {
                HTMLDocument.HTMLReader.this.blockClose(HTML.Tag.IMPLIED);
                HTMLDocument.HTMLReader.this.inPre = false;
                HTMLDocument.HTMLReader.this.blockClose(tag);
            }
        }
        
        public class CharacterAction extends HTMLDocument.HTMLReader.TagAction
        {
            @Override
            public void start(HTML.Tag tag, MutableAttributeSet set) {
                HTMLDocument.HTMLReader.this.pushCharacterStyle();
                if (!HTMLDocument.HTMLReader.this.foundInsertTag) {
                    int access$900 = HTMLDocument.HTMLReader.access$900(HTMLDocument.HTMLReader.this, tag, set, false);
                    if (HTMLDocument.HTMLReader.this.foundInsertTag && !HTMLDocument.HTMLReader.this.inParagraph) {
                        HTMLDocument.HTMLReader this$1 = HTMLDocument.HTMLReader.this;
                        HTMLDocument.HTMLReader this$2 = HTMLDocument.HTMLReader.this;
                        boolean b = true;
                        this$2.impliedP = b;
                        this$1.inParagraph = b;
                    }
                    if (access$900 == 0) {
                        return;
                    }
                }
                if (set.isDefined(HTMLEditorKit.ParserCallback.IMPLIED)) {
                    set.removeAttribute(HTMLEditorKit.ParserCallback.IMPLIED);
                }
                HTMLDocument.HTMLReader.this.charAttr.addAttribute(tag, set.copyAttributes());
                if (HTMLDocument.HTMLReader.this.styleAttributes != null) {
                    HTMLDocument.HTMLReader.this.charAttr.addAttributes(HTMLDocument.HTMLReader.this.styleAttributes);
                }
            }
            
            @Override
            public void end(HTML.Tag tag) {
                HTMLDocument.HTMLReader.this.popCharacterStyle();
            }
        }
        
        class ConvertAction extends HTMLDocument.HTMLReader.TagAction
        {
            @Override
            public void start(HTML.Tag tag, MutableAttributeSet set) {
                HTMLDocument.HTMLReader.this.pushCharacterStyle();
                if (!HTMLDocument.HTMLReader.this.foundInsertTag) {
                    int access$900 = HTMLDocument.HTMLReader.access$900(HTMLDocument.HTMLReader.this, tag, set, false);
                    if (HTMLDocument.HTMLReader.this.foundInsertTag && !HTMLDocument.HTMLReader.this.inParagraph) {
                        HTMLDocument.HTMLReader this$1 = HTMLDocument.HTMLReader.this;
                        HTMLDocument.HTMLReader this$2 = HTMLDocument.HTMLReader.this;
                        boolean b = true;
                        this$2.impliedP = b;
                        this$1.inParagraph = b;
                    }
                    if (access$900 == 0) {
                        return;
                    }
                }
                if (set.isDefined(HTMLEditorKit.ParserCallback.IMPLIED)) {
                    set.removeAttribute(HTMLEditorKit.ParserCallback.IMPLIED);
                }
                if (HTMLDocument.HTMLReader.this.styleAttributes != null) {
                    HTMLDocument.HTMLReader.this.charAttr.addAttributes(HTMLDocument.HTMLReader.this.styleAttributes);
                }
                HTMLDocument.HTMLReader.this.charAttr.addAttribute(tag, set.copyAttributes());
                StyleSheet styleSheet = HTMLDocument.this.getStyleSheet();
                if (tag == HTML.Tag.B) {
                    styleSheet.addCSSAttribute(HTMLDocument.HTMLReader.this.charAttr, CSS.Attribute.FONT_WEIGHT, "bold");
                }
                else if (tag == HTML.Tag.I) {
                    styleSheet.addCSSAttribute(HTMLDocument.HTMLReader.this.charAttr, CSS.Attribute.FONT_STYLE, "italic");
                }
                else if (tag == HTML.Tag.U) {
                    Object attribute4 = HTMLDocument.HTMLReader.this.charAttr.getAttribute(CSS.Attribute.TEXT_DECORATION);
                    String str4 = "underline";
                    styleSheet.addCSSAttribute(HTMLDocument.HTMLReader.this.charAttr, CSS.Attribute.TEXT_DECORATION, (attribute4 != null) ? (str4 + "," + attribute4.toString()) : str4);
                }
                else if (tag == HTML.Tag.STRIKE) {
                    Object attribute3 = HTMLDocument.HTMLReader.this.charAttr.getAttribute(CSS.Attribute.TEXT_DECORATION);
                    String str3 = "line-through";
                    styleSheet.addCSSAttribute(HTMLDocument.HTMLReader.this.charAttr, CSS.Attribute.TEXT_DECORATION, (attribute3 != null) ? (str3 + "," + attribute3.toString()) : str3);
                }
                else if (tag == HTML.Tag.SUP) {
                    Object attribute2 = HTMLDocument.HTMLReader.this.charAttr.getAttribute(CSS.Attribute.VERTICAL_ALIGN);
                    String str2 = "sup";
                    styleSheet.addCSSAttribute(HTMLDocument.HTMLReader.this.charAttr, CSS.Attribute.VERTICAL_ALIGN, (attribute2 != null) ? (str2 + "," + attribute2.toString()) : str2);
                }
                else if (tag == HTML.Tag.SUB) {
                    Object attribute = HTMLDocument.HTMLReader.this.charAttr.getAttribute(CSS.Attribute.VERTICAL_ALIGN);
                    String str = "sub";
                    styleSheet.addCSSAttribute(HTMLDocument.HTMLReader.this.charAttr, CSS.Attribute.VERTICAL_ALIGN, (attribute != null) ? (str + "," + attribute.toString()) : str);
                }
                else if (tag == HTML.Tag.FONT) {
                    String s = (String)set.getAttribute(HTML.Attribute.COLOR);
                    if (s != null) {
                        styleSheet.addCSSAttribute(HTMLDocument.HTMLReader.this.charAttr, CSS.Attribute.COLOR, s);
                    }
                    String s2 = (String)set.getAttribute(HTML.Attribute.FACE);
                    if (s2 != null) {
                        styleSheet.addCSSAttribute(HTMLDocument.HTMLReader.this.charAttr, CSS.Attribute.FONT_FAMILY, s2);
                    }
                    String s3 = (String)set.getAttribute(HTML.Attribute.SIZE);
                    if (s3 != null) {
                        styleSheet.addCSSAttributeFromHTML(HTMLDocument.HTMLReader.this.charAttr, CSS.Attribute.FONT_SIZE, s3);
                    }
                }
            }
            
            @Override
            public void end(HTML.Tag tag) {
                HTMLDocument.HTMLReader.this.popCharacterStyle();
            }
        }
        
        class AnchorAction extends HTMLDocument.HTMLReader.CharacterAction
        {
            @Override
            public void start(HTML.Tag tag, MutableAttributeSet set) {
                HTMLDocument.HTMLReader.this.emptyAnchor = true;
                super.start(tag, set);
            }
            
            @Override
            public void end(HTML.Tag tag) {
                if (HTMLDocument.HTMLReader.this.emptyAnchor) {
                    HTMLDocument.HTMLReader.this.addContent(new char[] { '\n' }, 0, 1);
                }
                super.end(tag);
            }
        }
        
        class TitleAction extends HTMLDocument.HTMLReader.HiddenAction
        {
            @Override
            public void start(HTML.Tag tag, MutableAttributeSet set) {
                HTMLDocument.HTMLReader.this.inTitle = true;
                super.start(tag, set);
            }
            
            @Override
            public void end(HTML.Tag tag) {
                HTMLDocument.HTMLReader.this.inTitle = false;
                super.end(tag);
            }
            
            @Override
            boolean isEmpty(HTML.Tag tag) {
                return false;
            }
        }
        
        class BaseAction extends HTMLDocument.HTMLReader.TagAction
        {
            @Override
            public void start(HTML.Tag tag, MutableAttributeSet set) {
                String spec = (String)set.getAttribute(HTML.Attribute.HREF);
                if (spec != null) {
                    try {
                        HTMLDocument.this.setBase(new URL(HTMLDocument.this.base, spec));
                        HTMLDocument.this.hasBaseTag = true;
                    }
                    catch (MalformedURLException ex) {}
                }
                HTMLDocument.this.baseTarget = (String)set.getAttribute(HTML.Attribute.TARGET);
            }
        }
        
        class ObjectAction extends HTMLDocument.HTMLReader.SpecialAction
        {
            @Override
            public void start(HTML.Tag tag, MutableAttributeSet set) {
                if (tag == HTML.Tag.PARAM) {
                    this.addParameter(set);
                }
                else {
                    super.start(tag, set);
                }
            }
            
            @Override
            public void end(HTML.Tag tag) {
                if (tag != HTML.Tag.PARAM) {
                    super.end(tag);
                }
            }
            
            void addParameter(AttributeSet set) {
                String s = (String)set.getAttribute(HTML.Attribute.NAME);
                String s2 = (String)set.getAttribute(HTML.Attribute.VALUE);
                if (s != null && s2 != null) {
                    ((MutableAttributeSet)((DefaultStyledDocument.ElementSpec)HTMLDocument.HTMLReader.this.parseBuffer.lastElement()).getAttributes()).addAttribute(s, s2);
                }
            }
        }
        
        public class FormAction extends HTMLDocument.HTMLReader.SpecialAction
        {
            Object selectModel;
            int optionCount;
            
            @Override
            public void start(HTML.Tag tag, MutableAttributeSet set) {
                if (tag == HTML.Tag.INPUT) {
                    String s = (String)set.getAttribute(HTML.Attribute.TYPE);
                    if (s == null) {
                        s = "text";
                        set.addAttribute(HTML.Attribute.TYPE, "text");
                    }
                    this.setModel(s, set);
                }
                else if (tag == HTML.Tag.TEXTAREA) {
                    HTMLDocument.HTMLReader.this.inTextArea = true;
                    HTMLDocument.HTMLReader.this.textAreaDocument = new TextAreaDocument();
                    set.addAttribute(StyleConstants.ModelAttribute, HTMLDocument.HTMLReader.this.textAreaDocument);
                }
                else if (tag == HTML.Tag.SELECT) {
                    int integerAttributeValue = HTML.getIntegerAttributeValue(set, HTML.Attribute.SIZE, 1);
                    int n = set.getAttribute(HTML.Attribute.MULTIPLE) != null;
                    if (integerAttributeValue > 1 || n) {
                        OptionListModel<Object> selectModel = new OptionListModel();
                        if (!(n == 0)) {
                            selectModel.setSelectionMode(2);
                        }
                        this.selectModel = selectModel;
                    }
                    else {
                        this.selectModel = new OptionComboBoxModel();
                    }
                    set.addAttribute(StyleConstants.ModelAttribute, this.selectModel);
                }
                if (tag == HTML.Tag.OPTION) {
                    HTMLDocument.HTMLReader.this.option = new Option(set);
                    if ((this.selectModel instanceof OptionListModel)) {
                        OptionListModel<HTMLDocument.HTMLReader> optionListModel = (OptionListModel)this.selectModel;
                        optionListModel.addElement(HTMLDocument.HTMLReader.this.option);
                        if (HTMLDocument.HTMLReader.this.option.isSelected()) {
                            optionListModel.addSelectionInterval(this.optionCount, this.optionCount);
                            optionListModel.setInitialSelection(this.optionCount);
                        }
                    }
                    else if ((this.selectModel instanceof OptionComboBoxModel)) {
                        OptionComboBoxModel<HTMLDocument.HTMLReader> optionComboBoxModel = (OptionComboBoxModel)this.selectModel;
                        optionComboBoxModel.addElement(HTMLDocument.HTMLReader.this.option);
                        if (HTMLDocument.HTMLReader.this.option.isSelected()) {
                            optionComboBoxModel.setSelectedItem(HTMLDocument.HTMLReader.this.option);
                            optionComboBoxModel.setInitialSelection(HTMLDocument.HTMLReader.this.option);
                        }
                    }
                    ++this.optionCount;
                }
                else {
                    super.start(tag, set);
                }
            }
            
            @Override
            public void end(HTML.Tag tag) {
                if (tag == HTML.Tag.OPTION) {
                    HTMLDocument.HTMLReader.this.option = null;
                }
                else {
                    if (tag == HTML.Tag.SELECT) {
                        this.selectModel = null;
                        this.optionCount = 0;
                    }
                    else if (tag == HTML.Tag.TEXTAREA) {
                        HTMLDocument.HTMLReader.this.inTextArea = false;
                        HTMLDocument.HTMLReader.this.textAreaDocument.storeInitialText();
                    }
                    super.end(tag);
                }
            }
            
            void setModel(String s, MutableAttributeSet set) {
                if (s.equals("submit") || s.equals("reset") || s.equals("image")) {
                    set.addAttribute(StyleConstants.ModelAttribute, new DefaultButtonModel());
                }
                else if (s.equals("text") || s.equals("password")) {
                    int integerAttributeValue = HTML.getIntegerAttributeValue(set, HTML.Attribute.MAXLENGTH, -1);
                    HTMLDocument.FixedLengthDocument fixedLengthDocument;
                    if (integerAttributeValue > 0) {
                        fixedLengthDocument = new FixedLengthDocument(integerAttributeValue);
                    }
                    else {
                        fixedLengthDocument = new PlainDocument();
                    }
                    String s3 = (String)set.getAttribute(HTML.Attribute.VALUE);
                    try {
                        fixedLengthDocument.insertString(0, s3, null);
                    }
                    catch (BadLocationException ex) {}
                    set.addAttribute(StyleConstants.ModelAttribute, fixedLengthDocument);
                }
                else if (s.equals("file")) {
                    set.addAttribute(StyleConstants.ModelAttribute, new PlainDocument());
                }
                else if (s.equals("checkbox") || s.equals("radio")) {
                    JToggleButton.ToggleButtonModel toggleButtonModel = new JToggleButton.ToggleButtonModel();
                    if (s.equals("radio")) {
                        String s2 = (String)set.getAttribute(HTML.Attribute.NAME);
                        if (HTMLDocument.this.radioButtonGroupsMap == null) {
                            HTMLDocument.this.radioButtonGroupsMap = new HashMap();
                        }
                        ButtonGroup buttonGroup = (ButtonGroup)HTMLDocument.this.radioButtonGroupsMap.get(s2);
                        if (buttonGroup == null) {
                            buttonGroup = new ButtonGroup();
                            HTMLDocument.this.radioButtonGroupsMap.put(s2, buttonGroup);
                        }
                        toggleButtonModel.setGroup(buttonGroup);
                    }
                    toggleButtonModel.setSelected(set.getAttribute(HTML.Attribute.CHECKED) != null);
                    set.addAttribute(StyleConstants.ModelAttribute, toggleButtonModel);
                }
            }
        }
    }
    
    static class TaggedAttributeSet extends SimpleAttributeSet
    {
    }
    
    public class RunElement extends AbstractDocument.LeafElement
    {
        public RunElement(Element parent, AttributeSet a, int offs0, int offs2) {
            super(parent, a, offs0, offs2);
        }
        
        @Override
        public String getName() {
            Object attribute = this.getAttribute(StyleConstants.NameAttribute);
            if (attribute != null) {
                return attribute.toString();
            }
            return super.getName();
        }
        
        @Override
        public AttributeSet getResolveParent() {
            return null;
        }
    }
    
    public class BlockElement extends AbstractDocument.BranchElement
    {
        public BlockElement(Element parent, AttributeSet a) {
            super(parent, a);
        }
        
        @Override
        public String getName() {
            Object attribute = this.getAttribute(StyleConstants.NameAttribute);
            if (attribute != null) {
                return attribute.toString();
            }
            return super.getName();
        }
        
        @Override
        public AttributeSet getResolveParent() {
            return null;
        }
    }
    
    private static class FixedLengthDocument extends PlainDocument
    {
        private int maxLength;
        
        public FixedLengthDocument(int maxLength) {
            this.maxLength = maxLength;
        }
        
        @Override
        public void insertString(int offs, String str, AttributeSet a) throws BadLocationException {
            if (str != null && str.length() + this.getLength() <= this.maxLength) {
                super.insertString(offs, str, a);
            }
        }
    }
}
